"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Path = require("path");
const glob = require("glob");
const imageDiff = require("./image-diff");
const rectangles = require("./rectangles");
const types_1 = require("./types");
const crypto = require("crypto");
const fs = require("fs");
const png_1 = require("./png");
function getFilePairs(file1, file2) {
    const name1 = `${Path.relative(".", file1)}`;
    const name2 = `${Path.relative(".", file2)}`;
    return {
        [`${name1} - ${name2}`]: {
            left: file1,
            right: file2
        }
    };
}
exports.getFilePairs = getFilePairs;
function getFilePairsRecursively(dir1, dir2) {
    const pattern = "**/*.png";
    const filePairs = {};
    collectFilePairs(filePairs, pattern, dir1, "left");
    collectFilePairs(filePairs, pattern, dir2, "right");
    return filePairs;
}
exports.getFilePairsRecursively = getFilePairsRecursively;
function collectFilePairs(filePairs, pattern, dir, field) {
    const files = glob.sync(pattern, {
        cwd: dir
    });
    for (let file of files) {
        filePairs[file] = filePairs[file] || {};
        filePairs[file][field] = Path.resolve(dir, file);
    }
}
function compareFile(info, options, logger) {
    return __awaiter(this, void 0, void 0, function* () {
        let leftName = "nothing";
        let rightName = "nothing";
        if (info.left) {
            leftName = Path.basename(info.left);
        }
        if (info.right) {
            rightName = Path.basename(info.right);
        }
        if (info.left && info.right && leftName === rightName) {
            leftName = Path.relative(".", info.left);
            rightName = Path.relative(".", info.right);
        }
        logger.log("comparing " + leftName + " and " + rightName);
        if (info.left && info.right && isHashEqual(info.left, info.right)) {
            logger.verbose("hash matched");
            return new types_1.FileDiff(null, null, { left: [], right: [] });
        }
        const left = info.left ? new png_1.Png(info.left) : null;
        const right = info.right ? new png_1.Png(info.right) : null;
        const leftInfo = info.left && {
            path: info.left,
            width: left.width,
            height: left.height
        };
        const rightInfo = info.right && {
            path: info.right,
            width: right.width,
            height: right.height
        };
        let results = [];
        if (left && right) {
            logger.verbose(`  left: ${leftInfo.width} * ${leftInfo.height}`);
            logger.verbose(`  right: ${rightInfo.width} * ${rightInfo.height}`);
            let start = Date.now();
            results = imageDiff.compareImage(left, right, options);
            logger.verbose("  took " + (Date.now() - start) + " ms to compare");
        }
        const change = {
            left: leftInfo,
            right: rightInfo,
            results
        };
        const start = Date.now();
        const rectsLR = yield rectangles.getRects(change, options);
        logger.verbose("  took " + (Date.now() - start) + " ms to get rectangles");
        return new types_1.FileDiff(change.left, change.right, rectsLR);
    });
}
exports.compareFile = compareFile;
function isHashEqual(left, right) {
    return md5file(left) === md5file(right);
}
function md5file(filePath) {
    const target = fs.readFileSync(filePath);
    const md5hash = crypto.createHash("md5");
    md5hash.update(target);
    return md5hash.digest("hex");
}
//# sourceMappingURL=files.js.map